<?php

/**
 * Cryptocurrency Payment Module
 *
 * @package paymentMethod
 * @copyright Copyright 2003-2018 Zen Cart Development Team
 * @copyright Portions Copyright 2003 osCommerce
 * @license http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version $Id: Drbyte Sun Jan 7 21:30:21 2018 -0500 Modified in v1.5.6 $
 */

class bropay extends base {
    var $code, $title, $description, $enabled;

    function __construct() {
        global $order;
        $this->code = 'bropay';
        $this->title = MODULE_PAYMENT_BROPAY_TEXT_TITLE;
        $this->description = MODULE_PAYMENT_BROPAY_TEXT_DESCRIPTION;
        $this->sort_order = defined('MODULE_PAYMENT_BROPAY_SORT_ORDER') ? MODULE_PAYMENT_BROPAY_SORT_ORDER : null;
        $this->enabled = (defined('MODULE_PAYMENT_BROPAY_STATUS') && MODULE_PAYMENT_BROPAY_STATUS == 'True');

        if (defined('MODULE_PAYMENT_BROPAY_ORDER_STATUS_ID') && (int)MODULE_PAYMENT_BROPAY_ORDER_STATUS_ID > 0) {
            $this->order_status = MODULE_PAYMENT_BROPAY_ORDER_STATUS_ID;
        }

        $this->merchant = defined('MODULE_PAYMENT_BROPAY_MERCHANT_ID')?MODULE_PAYMENT_BROPAY_MERCHANT_ID:null;


        if (is_object($order)) $this->update_status();
        if(!empty($order->info['total'])){
            $this->totalsum = $order->info['total'] = zen_round($order->info['total'], 2);
        }

        if(defined('MODULE_PAYMENT_BROPAY_DEBUG_EMAIL_ADDRESS')){
            $this->debug_email = MODULE_PAYMENT_BROPAY_DEBUG_EMAIL_ADDRESS;
        }
    }

    function update_status() {
        global $order, $db;

        if ($this->enabled && (int)MODULE_PAYMENT_BROPAY_ZONE > 0 && isset($order->delivery['country']['id'])) {
            $check_flag = false;
            $check = $db->Execute("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_BROPAY_ZONE . "' and zone_country_id = '" . (int)$order->delivery['country']['id'] . "' order by zone_id");
            while (!$check->EOF) {
                if ($check->fields['zone_id'] < 1) {
                    $check_flag = true;
                    break;
                } elseif ($check->fields['zone_id'] == $order->delivery['zone_id']) {
                    $check_flag = true;
                    break;
                }
                $check->MoveNext();
            }

            if ($check_flag == false) {
                $this->enabled = false;
            }
        }

        if ($this->enabled == true) {
            if ($order->content_type != 'physical') {
                $this->enabled = false;
            }
        }

        // other status checks?
        if ($this->enabled) {
            // other checks here
        }
    }

    function javascript_validation() {
        return false;
    }

    function selection() {
        return array('id' => $this->code,
            'module' => $this->title);
    }

    function pre_confirmation_check() {
        return false;
    }

    function confirmation() {
        return false;
    }

    function select_cp_currency(){
        if (MODULE_PAYMENT_BROPAY_CURRENCY == 'Selected Currency') {
            $my_currency = $_SESSION['currency'];
        } else {
            $my_currency = substr(MODULE_PAYMENT_BROPAY_CURRENCY, 5);
        }
        $pp_currencies = array('CAD', 'EUR', 'GBP', 'USD', 'AUD');
        if (!in_array($my_currency, $pp_currencies)) {
            $my_currency = 'USD';
        }
        return $my_currency;
    }

    function process_button() {
        return false;
    }

    function before_process() {
        return false;
    }

    function after_process() {
        global $order, $currencies;

        $order_number = $_SESSION['order_summary']['order_number'];

        $this->totalsum = $order->info['total'] = zen_round($order->info['total'], 2);
        $my_currency = $this->select_cp_currency();
        $this->transaction_currency = $my_currency;
        $this->transaction_amount = zen_round($this->totalsum * $currencies->get_value($my_currency), $currencies->get_decimal_places($my_currency));

        foreach($order->products as $item){
            $ordered[] = [
                "name" => $item['name'],
                "price" => $item['price'],
                "sku" => "sku".$item['id'],
                "qty"  => $item['qty']
            ];
        }

        $fields = array(
            'cmd' => '_pay_auto',
            'reset' => '1',
            'merchant' => $this->merchant,
            'item_name' => MODULE_PAYMENT_BROPAY_TEXT_DESCRIPTION,
            'invoice' => $order_number,
            'custom' => 3,
            'quantity' => '1',
            'gateway' => MODULE_PAYMENT_BROPAY_GATEWAY,
            'allow_quantity' => 0,
            'want_shipping' => 1,
            'shippingfirstname' => $order->delivery['firstname'],
            'shippinglastname' => $order->delivery['lastname'],
            'shippingaddress' => replace_accents($order->delivery['street_address'] != '' ? $order->delivery['street_address'] : $order->billing['street_address']),
            'shippingcity' => replace_accents($order->delivery['city'] != '' ? $order->delivery['city'] : $order->billing['city']),
            'shippingcountry' => ($order->delivery['country']['iso_code_2'] != '' ? $order->delivery['country']['iso_code_2'] : $order->billing['country']['iso_code_2']),
            'shippingzipcode' => ($order->delivery['postcode'] != '' ? $order->delivery['postcode'] : $order->billing['postcode']),
            'shippingtelephone' => preg_replace('/\D/', '', $order->customer['telephone']),
            'shippingstate' => ($order->delivery['country']['id'] != '' ? zen_get_zone_code($order->delivery['country']['id'], $order->delivery['zone_id'], $order->delivery['state']) : zen_get_zone_code($order->billing['country']['id'], $order->billing['zone_id'], $order->billing['state'])),
            'shippingstatecode' => ($order->delivery['country']['id'] != '' ? zen_get_zone_code($order->delivery['country']['id'], $order->delivery['zone_id'], $order->delivery['state']) : zen_get_zone_code($order->billing['country']['id'], $order->billing['zone_id'], $order->billing['state'])),
            'currency' => $this->transaction_currency,
            'shippingf' => '0.00',
            'taxf' => '0.00',
            'amountf' => $this->totalsum,
            'email' => $order->customer['email_address'],
            'first_name' => $order->delivery['firstname'],
            'last_name' => $order->delivery['lastname'],
            'success_url' => zen_href_link(FILENAME_CHECKOUT_SUCCESS, 'referer=bropay', 'SSL'),
            'cancel_url' => zen_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL'),
            'ipn_url' => zen_href_link('ipn_bropay.php', '', 'SSL',false,false,true),
            'items' => serialize($ordered),
            'debug_email' => $this->debug_email
        );

        $arg_string = http_build_query($fields);
        zen_redirect(zen_href_link('ipn_bropay.php?'.$arg_string, '', 'SSL',false,false,true));
    }

    function get_error() {
        return false;
    }

    function check() {
        global $db;
        if (!isset($this->_check)) {
            $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_BROPAY_STATUS'");
            $this->_check = $check_query->RecordCount();
        }
        return $this->_check;
    }

    function install() {
        global $db, $messageStack;
        if (defined('MODULE_PAYMENT_BROPAY_STATUS')) {
            $messageStack->add_session('Cryptocurrency payment module already installed.', 'error');
            zen_redirect(zen_href_link(FILENAME_MODULES, 'set=payment&module=bropay', 'NONSSL'));
            return 'failed';
        }
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable Crypto Payment On Delivery Module', 'MODULE_PAYMENT_BROPAY_STATUS', 'True', 'Do you want to accept Cash On Delivery payments?', '6', '1', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Payment Zone', 'MODULE_PAYMENT_BROPAY_ZONE', '0', 'If a zone is selected, only enable this payment method for that zone.', '6', '2', 'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Merchant ID', 'MODULE_PAYMENT_BROPAY_MERCHANT_ID','', 'Your merchant ID.', '6', '2', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Payment Gateway', 'MODULE_PAYMENT_BROPAY_GATEWAY','', 'Gateway URL', '6', '2', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Debug Email Address', 'MODULE_PAYMENT_BROPAY_DEBUG_EMAIL_ADDRESS','', 'The email address to use for IPN debugging. (leave empty for no emails)', '6', '72', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Transaction Currency', 'MODULE_PAYMENT_BROPAY_CURRENCY', 'Selected Currency', 'Which currency should the order be sent to cryptocurrency as? <br />NOTE: if an unsupported currency is sent to cryptocurrency, it will be auto-converted to USD.', '6', '3', 'zen_cfg_select_option(array(\'Selected Currency\', \'Only USD\', \'Only AUD\', \'Only CAD\', \'Only EUR\', \'Only GBP\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_BROPAY_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '0', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Order Status', 'MODULE_PAYMENT_BROPAY_ORDER_STATUS_ID', '1', 'Set the status of orders made with this payment module to this value', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
    }

    function remove() {
        global $db;
        $db->Execute("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
    }

    function keys(){
        return array(
            'MODULE_PAYMENT_BROPAY_STATUS',
            'MODULE_PAYMENT_BROPAY_GATEWAY',
            'MODULE_PAYMENT_BROPAY_MERCHANT_ID',
            'MODULE_PAYMENT_BROPAY_DEBUG_EMAIL_ADDRESS',
            'MODULE_PAYMENT_BROPAY_CURRENCY',
            'MODULE_PAYMENT_BROPAY_ZONE',
            'MODULE_PAYMENT_BROPAY_ORDER_STATUS_ID',
            'MODULE_PAYMENT_BROPAY_SORT_ORDER'
        );
    }
}